/* GDK - The GIMP Drawing Kit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdkdirectfb.h"

#include "gdkcolor.h"


static void     gdk_colormap_init        (GdkColormap      *colormap);
static void     gdk_colormap_class_init  (GdkColormapClass *klass);
static void     gdk_colormap_finalize    (GObject          *object);

static gpointer parent_class = NULL;


GType
gdk_colormap_get_type (void)
{
  static GType object_type = 0;

  if (!object_type) {
    static const GTypeInfo object_info =
      {
        sizeof (GdkColormapClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) gdk_colormap_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (GdkColormap),
        0,              /* n_preallocs */
        (GInstanceInitFunc) gdk_colormap_init,
      };

    object_type = g_type_register_static (G_TYPE_OBJECT,
                                          "GdkColormap",
                                          &object_info, 0);
  }

  return object_type;
}

static void
gdk_colormap_init (GdkColormap *colormap)
{
  colormap->size = 0;
  colormap->colors = NULL;
}

static void
gdk_colormap_class_init (GdkColormapClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = gdk_colormap_finalize;
}

static void
gdk_colormap_finalize (GObject *object)
{
  GdkColormap *colormap = GDK_COLORMAP (object);

  g_free (colormap->colors);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}

GdkColormap*
gdk_colormap_new (GdkVisual *visual,
                  gboolean   private_cmap)
{
  GdkColormap *colormap;

  g_return_val_if_fail (visual != NULL, NULL);

  colormap = g_object_new (gdk_colormap_get_type (), NULL);

  colormap->visual = visual;
  colormap->size   = visual->colormap_size;

  return colormap;
}

void
gdk_colormap_sync (GdkColormap *colormap,
                   gboolean     force)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);
}

GdkColormap*
gdk_colormap_get_system (void)
{
  static GdkColormap *colormap = NULL;

  if (!colormap)
    colormap = gdk_colormap_new (gdk_visual_get_system(), FALSE);

  return colormap;
}


gint
gdk_colormap_get_system_size (void)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);

  return 65536;
}

void
gdk_colormap_change (GdkColormap *colormap,
             gint         ncolors)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);
}

gboolean
gdk_colors_alloc (GdkColormap   *colormap,
                  gboolean       contiguous,
                  gulong        *planes,
                  gint           nplanes,
                  gulong        *pixels,
                  gint           npixels)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);
   
  return FALSE;
}

void
gdk_colors_free (GdkColormap *colormap,
                 gulong      *in_pixels,
                 gint         in_npixels,
                 gulong       planes)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);
}

void
gdk_colormap_free_colors (GdkColormap *colormap,
                          GdkColor    *colors,
                          gint         ncolors)
{
}

gint
gdk_colormap_alloc_colors (GdkColormap *colormap,
                           GdkColor    *colors,
                           gint         ncolors,
                           gboolean     writeable,
                           gboolean     best_match,
                           gboolean    *success)
{
  GdkVisual *visual;
  gint i;

  g_return_val_if_fail (GDK_IS_COLORMAP (colormap), FALSE);
  g_return_val_if_fail (colors != NULL, FALSE);

  for (i = 0; i < ncolors; i++)
    {
      success[i] = FALSE;
    }
  
  switch (colormap->visual->type)
    {
    case GDK_VISUAL_PSEUDO_COLOR:
    case GDK_VISUAL_GRAYSCALE:
      //TODODO
      break;
      
    case GDK_VISUAL_TRUE_COLOR:
      visual = colormap->visual;
      
      for (i = 0; i < ncolors; i++)
        {
          colors[i].pixel = (((colors[i].red  >> (16 - visual->red_prec)) << visual->red_shift) +
                             ((colors[i].green >> (16 - visual->green_prec)) << visual->green_shift) +
                             ((colors[i].blue  >> (16 - visual->blue_prec)) << visual->blue_shift));
          success[i] = TRUE;
        }
      break;
      
    case GDK_VISUAL_STATIC_GRAY:
    case GDK_VISUAL_STATIC_COLOR:
      //TODODO
      break;
      
    case GDK_VISUAL_DIRECT_COLOR:
      g_assert_not_reached ();
    }

  return 0;
}

gboolean
gdk_color_change (GdkColormap *colormap,
                  GdkColor    *color)
{
  g_message("unimplemented %s", G_GNUC_FUNCTION);

  return TRUE;
}

void
gdk_colormap_query_color (GdkColormap *colormap,
                          gulong       pixel,
                          GdkColor    *result)
{
  GdkVisual *visual;

  g_return_if_fail (GDK_IS_COLORMAP (colormap));
  
  visual = gdk_colormap_get_visual (colormap);

  switch (visual->type) 
    {
    case GDK_VISUAL_DIRECT_COLOR:
    case GDK_VISUAL_TRUE_COLOR:
      result->red = 65535. *
                   (gdouble)((pixel & visual->red_mask) >> visual->red_shift) /
                   ((1 << visual->red_prec) - 1);
      
      result->green = 65535. *
                     (gdouble)((pixel & visual->green_mask) >> visual->green_shift) /
                     ((1 << visual->green_prec) - 1);
      
      result->blue = 65535. *
                    (gdouble)((pixel & visual->blue_mask) >> visual->blue_shift) /
                    ((1 << visual->blue_prec) - 1);
      break;
      
    case GDK_VISUAL_STATIC_GRAY:
    case GDK_VISUAL_GRAYSCALE:
      result->red   =
      result->green =
      result->blue  = 65535. * (double)pixel/((1<<visual->depth) - 1);
      break;
      
    case GDK_VISUAL_PSEUDO_COLOR:
      result->red   = colormap->colors[pixel].red;
      result->green = colormap->colors[pixel].green;
      result->blue  = colormap->colors[pixel].blue;
      break;
      
    default:
      g_assert_not_reached ();
      break;
    }
}

